# load and register editors
from orangecontrib.snom.widgets.preprocessors import phase_unwrap  # noqa: F401
from orangecontrib.snom.widgets.preprocessors import background_fit  # noqa: F401
from orangecontrib.snom.widgets.preprocessors import phase_rotation  # noqa: F401
from orangecontrib.snom.widgets.preprocessors import simple_normalize  # noqa: F401
from orangecontrib.snom.widgets.preprocessors import linelevel  # noqa: F401
from orangecontrib.snom.widgets.preprocessors import self_reference  # noqa: F401
